Package: transmogR
Type: Package
Title: Modify a set of reference sequences using a set of variants
Version: 1.9.0
Authors@R: person("Stevie", "Pederson", 
    email = "stephen.pederson.au@gmail.com", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-8197-3303")
    )
Description: transmogR provides the tools needed to crate a new reference
    genome or reference transcriptome, using a set of variants. Variants can 
    be any combination of SNPs, Insertions and Deletions. The intended 
    use-case is to enable creation of variant-modified reference 
    transcriptomes for incorporation into transcriptomic pseudo-alignment 
    workflows, such as salmon.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/smped/transmogR
BugReports: https://github.com/smped/transmogR/issues
Depends: R (>= 4.1.0), Biostrings, GenomicRanges
Imports: BSgenome, data.table, Seqinfo, GenomicFeatures, ggplot2 (>=
        4.0.0), IRanges, jsonlite, matrixStats, methods, parallel,
        patchwork, scales, stats, S4Vectors, SummarizedExperiment,
        VariantAnnotation
Suggests: BiocStyle, BSgenome.Hsapiens.UCSC.hg38, edgeR, extraChIPs,
        InteractionSet, knitr, readr, rmarkdown, rtracklayer,
        SimpleUpset, testthat (>= 3.0.0)
biocViews: Alignment, GenomicVariation, Sequencing,
        TranscriptomeVariant, VariantAnnotation
BiocType: Software
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/transmogR
git_branch: devel
git_last_commit: 2035130
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 01:39:31 UTC; biocbuild
Author: Stevie Pederson [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8197-3303>)
Maintainer: Stevie Pederson <stephen.pederson.au@gmail.com>
