# Generated by roxygen2: do not edit by hand

export(cleanVariants)
export(digestSalmon)
export(genomogrify)
export(indelcator)
export(overdispFromBoots)
export(overlapsByVar)
export(owl)
export(parY)
export(shiftByVar)
export(sjFromExons)
export(transmogrify)
export(upsetVarByCol)
export(varTags)
export(varTypes)
exportMethods(cleanVariants)
exportMethods(genomogrify)
exportMethods(indelcator)
exportMethods(overlapsByVar)
exportMethods(owl)
exportMethods(transmogrify)
import(Biostrings)
import(GenomicRanges)
importClassesFrom(BSgenome,BSgenome)
importClassesFrom(SummarizedExperiment,SummarizedExperiment)
importClassesFrom(VariantAnnotation,ScanVcfParam)
importClassesFrom(VariantAnnotation,VcfFile)
importFrom(BSgenome,getSeq)
importFrom(Biostrings,IUPAC_CODE_MAP)
importFrom(Biostrings,getSeq)
importFrom(GenomicFeatures,extractTranscriptSeqs)
importFrom(IRanges,"width<-")
importFrom(IRanges,IRanges)
importFrom(IRanges,Views)
importFrom(IRanges,end)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,start)
importFrom(IRanges,subsetByOverlaps)
importFrom(IRanges,width)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,"seqlengths<-")
importFrom(Seqinfo,"seqlevels<-")
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqlevelsInUse)
importFrom(Seqinfo,seqnames)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,rowRanges)
importFrom(VariantAnnotation,"vcfWhich<-")
importFrom(VariantAnnotation,ScanVcfParam)
importFrom(VariantAnnotation,readVcf)
importFrom(data.table,fread)
importFrom(ggplot2,guide_none)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_fill_discrete)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowSums2)
importFrom(matrixStats,rowVars)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(parallel,mclapply)
importFrom(patchwork,plot_annotation)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,qf)
importFrom(stats,setNames)
useDynLib(transmogR, .registration = TRUE)
