library(rtracklayer)
library(VariantAnnotation)
library(SummarizedExperiment)
library(GenomicRanges)
library(patchwork)
library(ggplot2)
gtf <- import.gff(
    system.file("extdata/gencode.v44.subset.gtf.gz", package = "transmogR")
)
vcf <- system.file("extdata/1000GP_subset.vcf.gz", package = "transmogR")
var <- rowRanges(readVcf(vcf, param = ScanVcfParam(fixed = "ALT")))
